
float4x4 tWVP : WORLDVIEWPROJECTION;
float4x4 tWV : WORLDVIEW;

float d;
float color1;

struct vsInput
{
    float4 posObject : POSITION;
};

struct psInput
{
    float4 posScreen : SV_Position;
};

struct vsin
{
	float4 pos : POSITION;
	float4 norm : NORMAL;

};

struct psIn
{
    float4 pos: SV_Position;
    float4 norm: TEXCOORD0;
};

struct vs2gs
{
    float4 pos : POSITION;
	float4 norm : NORMAL;
 };

Texture2D inputTexture <string uiname="Texture";>;

SamplerState linearSampler <string uiname="Sampler State";>
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
};

cbuffer cbPerDraw : register(b0)
{
	float4x4 tVP : LAYERVIEWPROJECTION;
};

cbuffer cbPerObj : register( b1 )
{
	float4x4 tW : WORLD;
	float Alpha <float uimin=0.0; float uimax=1.0;> = 1; 
	float4 cAmb <bool color=true;String uiname="Color";> = { 1.0f,1.0f,1.0f,1.0f };
	float4x4 tColor <string uiname="Color Transform";>;
};

cbuffer cbTextureData : register(b2)
{
	float4x4 tTex <string uiname="Texture Transform"; bool uvspace=true; >;
};
// eto staroe ------------------------------
vs2gs VS(vsin input)
{
    vs2gs output;
	output.pos = input.pos;
	output.norm = input.norm;
    return output;
}
//------------------------------------------
/*
psInput VS(vsInput input)
{
	psInput output;
	output.posScreen = mul(input.posObject,mul(tW,tVP));
	return output;
}
*/
psIn VS_Textured(vsin input)
{
	psIn output;
	output.pos = mul(input.pos,mul(tW,tVP));
	output.norm = mul(input.norm, tTex);
	return output;
}

float4 PS(psInput input): SV_Target
{
    float4 col = cAmb;
	col = mul(col, tColor);
	col.a *= Alpha;
    return col;
}

[maxvertexcount(3)]
void GS(triangle vs2gs input[3], inout TriangleStream<psIn> gsout)
{
	psIn o;
	
	//Get triangle face direction
	float3 f1 = input[1].pos.xyz - input[0].pos.xyz;
    float3 f2 = input[2].pos.xyz - input[0].pos.xyz;
    
	//Compute flat normal
	float3 norm = normalize(cross(f1, f2));

	o.norm = float4(normalize(norm),1);

	float4 q = float4(norm *d, 0);
	
	//Transform triangles
	o.pos = mul(input[0].pos + q,tWVP);
	gsout.Append(o);
	
	o.pos = mul(input[1].pos + q,tWVP);
	gsout.Append(o);
	
	o.pos = mul(input[2].pos + q,tWVP);
	gsout.Append(o);
}


float4 PS_Textured(psIn input): SV_Target
{
    float4 col = inputTexture.Sample(linearSampler,input.norm.xy) * cAmb;
	col = mul(col, tColor);
	col.a *= Alpha;
    return col;
}

technique11 Constant <string noTexCdFallback="ConstantNoTexture"; >
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_4_0, VS_Textured() ) );
		SetPixelShader( CompileShader( ps_4_0, PS_Textured() ) );
	}
}

technique10 ConstantNoTexture
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_4_0, VS() ) );
		SetGeometryShader( CompileShader (gs_4_0, GS() )); 
		SetPixelShader( CompileShader( ps_4_0, PS() ) );
	}
}



